@@ -8,7 +8,7 @@ from typing import List, Tuple, Union
 from abc import ABC, abstractmethod
 from aenum import extend_enum
 from scipy.optimize import Bounds
-from commonroad.scenario.trajectory import State
+from commonroad.scenario.trajectory import State, CustomState
 from commonroad_rl.gym_commonroad.utils.scenario import make_valid_orientation
 from commonroad.common.solution import VehicleModel, VehicleType
 from commonroad_dc.collision.trajectory_queries import trajectory_queries
@@ -17,6 +17,7 @@ from vehiclemodels.vehicle_parameters import VehicleParameters
 from vehiclemodels.parameters_vehicle1 import parameters_vehicle1
 from vehiclemodels.parameters_vehicle2 import parameters_vehicle2
 from vehiclemodels.parameters_vehicle3 import parameters_vehicle3
+from scipy.integrate import odeint
 
 N_INTEGRATION_STEPS = 100
 
@@ -54,7 +55,7 @@ class Vehicle(ABC):
         """ Initialize empty object """
         vehicle_type = VehicleType(params_dict["vehicle_type"])
         vehicle_model = VehicleModel(params_dict["vehicle_model"])
-        
+
         self.vehicle_type = vehicle_type
         self.vehicle_model = assert_vehicle_model(vehicle_model)
         self.parameters = to_vehicle_parameter(vehicle_type)
@@ -151,8 +152,8 @@ class Vehicle(ABC):
         self.dt = dt
         if self.vehicle_model == VehicleModel.PM:
             orientation = initial_state.orientation if hasattr(initial_state, "orientation") else 0.0
-            self.initial_state = State(**{"position": initial_state.position,
-                                          "orientation": orientation,
+            self.initial_state = CustomState(**{"position": initial_state.position,
+                                          "orientation": make_valid_orientation(orientation),
                                           "time_step": initial_state.time_step,
                                           "velocity": initial_state.velocity * np.cos(orientation),
                                           "velocity_y": initial_state.velocity * np.sin(orientation),
@@ -161,14 +162,14 @@ class Vehicle(ABC):
                                           "acceleration_y": initial_state.acceleration * np.sin(orientation)
                                           if hasattr(initial_state, "acceleration") else 0.0})
         elif self.vehicle_model == VehicleModel.QP:
-            self.initial_state = State(**{"position": initial_state.position,
+            self.initial_state = CustomState(**{"position": initial_state.position,
                                           "velocity": initial_state.velocity,
                                           "acceleration": initial_state.acceleration
                                           if hasattr(initial_state, "acceleration")
                                           else 0.0,
                                           "jerk": initial_state.jerk
                                           if hasattr(initial_state, "jerk") else 0.0,
-                                          "orientation": initial_state.orientation
+                                          "orientation": make_valid_orientation(initial_state.orientation)
                                           if hasattr(initial_state, "orientation")
                                           else 0.0,
                                           "slip_angle": 0.0,
@@ -176,11 +177,11 @@ class Vehicle(ABC):
                                           "time_step": initial_state.time_step,
                                           })
         else:
-            self.initial_state = State(**{"position": initial_state.position,
+            self.initial_state = CustomState(**{"position": initial_state.position,
                                           "steering_angle": initial_state.steering_angle
                                           if hasattr(initial_state, "steering_angle")
                                           else 0.0,
-                                          "orientation": initial_state.orientation
+                                          "orientation": make_valid_orientation(initial_state.orientation)
                                           if hasattr(initial_state, "orientation")
                                           else 0.0,
                                           "yaw_rate": initial_state.yaw_rate
@@ -319,7 +320,12 @@ class ContinuousVehicle(Vehicle):
                 "time_step": current_state.time_step + 1,
             }
         elif self.vehicle_model == VehicleModel.QP:
-            return self.vehicle_dynamic.array_to_state(x_current, time_step=current_state.time_step + 1)
+            state = self.vehicle_dynamic.array_to_state(x_current, time_step=current_state.time_step + 1)
+            # TODO velocity_z and roll_rate are used to jounce and kappa_dot_dot
+            state.velocity_z = u_input[0]
+            state.roll_rate = u_input[1]
+
+            return state
         elif self.vehicle_model == VehicleModel.PMNonlinear:
             kwarg = {
                 "position": np.array([x_current[0], x_current[1]]),
@@ -330,7 +336,7 @@ class ContinuousVehicle(Vehicle):
                 "time_step": current_state.time_step + 1,
             }
 
-        return State(**kwarg)
+        return CustomState(**kwarg)
 
     def get_new_state(self, action: np.ndarray, action_base: str) -> State:
         """Generate the next state from current state for the given action.
@@ -449,7 +455,7 @@ class YawRateDynamics(VehicleDynamics):
             'velocity': x[3],
             'orientation': x[4],
         }
-        state = State(**values, time_step=time_step)
+        state = CustomState(**values, time_step=time_step)
         return state
 
     def _input_to_array(self, input: State) -> Tuple[np.array, int]:
@@ -471,7 +477,7 @@ class YawRateDynamics(VehicleDynamics):
             'yaw_rate': u[0],
             'acceleration': u[1],
         }
-        return State(**values, time_step=time_step)
+        return CustomState(**values, time_step=time_step)
 
 
 class QPDynamics(VehicleDynamics):
@@ -482,11 +488,120 @@ class QPDynamics(VehicleDynamics):
 
     def __init__(self, vehicle_type: VehicleType):
         super(QPDynamics, self).__init__(VehicleModel.QP, vehicle_type)
+        self.theta_ref = 0.
 
-    def dynamics(self, t, x, u) -> List[float]:
-        pass
+    @property
+    def input_bounds(self) -> Bounds:
+        """
+        Overrides the bounds method of Vehicle Model in order to return bounds for the Yaw Rate Model inputs.
 
-    def forward_simulation(self, x: np.array, u: np.array, dt: float, throw: bool = True) -> np.array:
+        Bounds are
+            - max jerk_dot <= jerk_dot <= max jerk_dot
+            - min kappa_dot_dot <= kappa_dot_dot <= max kappa_dot_dot
+
+        :return: Bounds
+        """
+        return Bounds(
+            np.array([-1000., -20.]),
+            np.array([1000., 20.])
+        )
+
+    @staticmethod
+    def jerk_dot_constraints(jerk_dot, acceleration, p):
+        if (jerk_dot < 0. and acceleration <= -p.a_max) or \
+                (jerk_dot > 0. and acceleration >= p.a_max):
+            jerk_dot = 0.
+        # TODO: integrate jerk dot constrain in vehicle parameters
+        return jerk_dot
+
+    @staticmethod
+    def kappa_dot_dot_constraints(kappa_dot_dot, kappa_dot, kappa_dot_min, kappa_dot_max):
+        if (kappa_dot < kappa_dot_min and kappa_dot_dot < 0.) or (kappa_dot > kappa_dot_max and kappa_dot_dot > 0.):
+            kappa_dot_dot = 0.
+        # TODO: integrate jerk dot constrain in vehicle parameters
+        return kappa_dot_dot
+
+    @staticmethod
+    def vehicle_dynamics_linear(x_init, u_init, kappa_dot_max, p, theta_ref):
+        x = np.array([x_init[:4], x_init[4:]])
+        u = []
+        u.append(QPDynamics.jerk_dot_constraints(u_init[0], x[0, 2], p.longitudinal))
+        u.append(QPDynamics.kappa_dot_dot_constraints(u_init[1], x[1, 3], -kappa_dot_max, kappa_dot_max))
+
+        x_long = x[0, :]
+        x_lat = x[1, :]
+
+        # longitudinal dynamics
+        f_long = [
+            x_long[1],
+            x_long[2],
+            x_long[3],
+            u[0]
+        ]
+
+        # lateral dynamics
+        v = x_long[1]  # TODO: QP planner uses the propagated velocity
+
+        f_lat = [
+            v * x_lat[1] - v * theta_ref,
+            v * x_lat[2],
+            x_lat[3],
+            u[1]
+        ]
+
+        # f = [f_long, f_lat]
+        f = f_long + f_lat
+
+        return f
+
+    def discrete_dynamics(self, x, u_init, dt, kappa_dot_max):
+        x = np.array([x[:4], x[4:]])
+        u = []
+        u.append(QPDynamics.jerk_dot_constraints(u_init[0], x[0, 2], self.parameters.longitudinal))
+        u.append(QPDynamics.kappa_dot_dot_constraints(u_init[1], x[1, 3], -kappa_dot_max, kappa_dot_max))
+
+        x_long = x[0, :]
+        x_lat = x[1, :]
+
+        A_long = np.array(
+            [[1, dt, (dt ** 2.) / 2., (dt ** 3.) / 6.],
+             [0, 1., dt, (dt ** 2.) / 2.],
+             [0, 0, 1., dt],
+             [0, 0, 0, 1]]
+        )
+        B_long = np.array([[(dt ** 4.) / 24.], [(dt ** 3.) / 6.], [(dt ** 2.) / 2.], [dt]])
+
+        v = x_long[1]
+        A_lat = np.array(
+            [
+                [1, dt * v, (dt ** 2) * 0.5 * (v ** 2), (dt ** 3) / 6 * (v ** 2)],
+                [0, 1, dt * v, (dt ** 2) * 0.5 * v],
+                [0, 0, 1, dt],
+                [0, 0, 0, 1]
+            ])
+
+        B_lat = np.array([[(dt ** 4) / 24 * (v ** 2)],
+                          [(dt ** 3) / 6 * v],
+                          [(dt ** 2) * 0.5],
+                          [dt]])
+
+        x_long = np.dot(A_long, x_long) + np.squeeze(B_long * u[0])
+        x_lat = np.dot(A_lat, x_lat) + np.squeeze(B_lat * u[1])
+
+        return np.concatenate([x_long, x_lat])
+
+    def dynamics(self, t, x, u_init, dt) -> List[float]:
+        # wb = self.parameters.a + self.parameters.b
+        # kappa_max = np.tan(self.parameters.steering.max) / wb
+        kappa_max = 0.2
+        kappa_dot_max = 2 * kappa_max / dt
+        self.theta_ref = 0.  # TODO: integrate reference path
+        return QPDynamics.vehicle_dynamics_linear(x, u_init, kappa_dot_max, self.parameters, self.theta_ref)
+
+        # # discrete integration
+        # return self.discrete_dynamics(x, u_init, dt, kappa_dot_max)
+
+    def forward_simulation(self, x: np.array, u_init: np.array, dt: float, throw: bool = True) -> np.array:
         """
         Simulates the next state using the given state and input values as numpy arrays.
 
@@ -496,103 +611,42 @@ class QPDynamics(VehicleDynamics):
         :param throw: if set to false, will return None as next state instead of throwing exception (default=True)
         :return: simulated next state values, raises VehicleDynamicsException if invalid input.
         """
-        # longitudinal x: (s, v, a, j)
-        A_long = np.array(
-            [[1, dt, (dt ** 2.) / 2., (dt ** 3.) / 6.],
-             [0, 1., dt, (dt ** 2.) / 2.],
-             [0, 0, 1., dt],
-             [0, 0, 0, 1]])  # 4x4
-        B_long = np.array([[(dt ** 4.) / 24.], [(dt ** 3.) / 6.], [(dt ** 2.) / 2.], [dt]])  # 4x1
-        x1_long = np.dot(A_long, x[0, :]) + np.squeeze(np.dot(B_long, u[0]))  # j_dot
 
-        j_max = 2*self.parameters.longitudinal.a_max / dt
+        x0, x1 = odeint(self.dynamics, x, [0.0, dt], args=(u_init, dt, ), tfirst=True)
+        # x1 = self.dynamics(None, x, u_init, dt)
 
-        if x1_long[2] > self.parameters.longitudinal.a_max:
-            x1_long[2] = self.parameters.longitudinal.a_max
-            if x1_long[3] > 0:
-                x1_long[3] = 0
-        if x1_long[2] < -self.parameters.longitudinal.a_max:
-            x1_long[2] = -self.parameters.longitudinal.a_max
-            if x1_long[3] < 0:
-                x1_long[3] = 0
-
-        if x1_long[3] > j_max:
-            x1_long[3] = j_max
-            if u[0] > 0:
-                u[0] = 0
-        if x1_long[3] < -j_max:
-            x1_long[3] = -j_max
-            if u[0] < 0:
-                u[0] = 0
-
-
-        # lateral x: (d, θ, κ, ˙κ)
-        v = x1_long[1]
-        theta = 0.0  # x_ref.reference[k].theta
-
-        # x = Ax+Bu+Dz
-        A_lat = np.array(
-            [[1, dt * v, (dt ** 2) * 0.5 * (v ** 2), (dt ** 3) / 6 * (v ** 2)],
-             [0, 1, dt * v, (dt ** 2) * 0.5 * v],
-             [0, 0, 1, dt],
-             [0, 0, 0, 1]])
-        B_lat = np.array([[(dt ** 4) / 24 * (v ** 2)],
-                          [(dt ** 3) / 6 * v],
-                          [(dt ** 2) * 0.5],
-                          [dt]])
-        # disturbances on input
-        D = np.array([-dt * v, 0, 0, 0])
-
-        x1_lat = np.dot(A_lat, x[1, :]) + np.squeeze(np.dot(B_lat, u[1])) + np.dot(theta, D)  # kappa_dot_dot
-
-        wb = self.parameters.a + self.parameters.b
-        # kappa_max = np.tan(self.parameters.steering.max) / wb
-        kappa_max = 0.2
-        kappa_dot_max = 2*kappa_max / dt
-
-        if x1_lat[2] > kappa_max:
-            x1_lat[2] = kappa_max
-            if x1_lat[3] > 0:
-                x1_lat[3] = 0.
-        if x1_lat[2] < -kappa_max:
-            x1_lat[2] = -kappa_max
-            if x1_lat[3] < 0:
-                x1_lat[3] = 0.
-
-        if x1_lat[3] > kappa_dot_max:
-            x1_lat[3] = kappa_dot_max
-        if x1_lat[3] < -kappa_dot_max:
-            x1_lat[3] = -kappa_dot_max
-
-        return np.array([x1_long, x1_lat])
+        return x1
 
     def _state_to_array(self, state: State, steering_angle_default=0.0) -> Tuple[np.array, int]:
         """ Implementation of the VehicleDynamics abstract method. """
         # (s, v, a, j)  (d, θ, κ,˙κ)
+        theta_ref = 0.
         # TODO: convert Cartesian position to CCOSY position
-        values = np.array([
-            [state.position[0] - self.parameters.b * np.cos(state.orientation), state.velocity, state.acceleration,
-             state.jerk],
+        values = np.array(
+            [state.position[0] - self.parameters.b * np.cos(state.orientation),
+             state.velocity * np.cos(state.orientation - self.theta_ref), state.acceleration, state.jerk] +
             [state.position[1] - self.parameters.b * np.sin(state.orientation), state.orientation, state.slip_angle,
              state.yaw_rate]
-        ])  # TODO slip_angle and yaw_rate are used to store kappa and kappa_dot
+        )  # TODO slip_angle and yaw_rate are used to store kappa and kappa_dot
 
         return values, state.time_step
 
-    def _array_to_state(self, x: np.array, time_step: int) -> State:
+    def _array_to_state(self, x_init: np.array, time_step: int) -> State:
         """ Implementation of the VehicleDynamics abstract method. """
         # TODO: convert CCOSY position to Cartesian position
+        # (s, v, a, j)  (d, θ, κ,˙κ)
+        x = np.array([x_init[:4], x_init[4:]])
         values = {
             'position': np.array([x[0, 0] + self.parameters.b * np.cos(x[1, 1]),
                                   x[1, 0] + self.parameters.b * np.sin(x[1, 1])]),
-            'velocity': x[0, 1],
+            'velocity': x[0, 1] / np.cos(x[1, 1] - self.theta_ref),
             'acceleration': x[0, 2],
             'jerk': x[0, 3],
             'orientation': make_valid_orientation(x[1, 1]),
             'slip_angle': x[1, 2],
-            'yaw_rate': x[1, 3]
+            'yaw_rate': x[1, 3],
         }
-        state = State(**values, time_step=time_step)
+        state = CustomState(**values, time_step=time_step)
         return state
 
 
@@ -630,7 +684,7 @@ class PointMassNonlinearDynamics(VehicleDynamics):
         :return: Bounds
         """
         return Bounds([-self.parameters.longitudinal.a_max, -self.parameters.longitudinal.a_max],
-            [self.parameters.longitudinal.a_max, self.parameters.longitudinal.a_max])
+                      [self.parameters.longitudinal.a_max, self.parameters.longitudinal.a_max])
 
     def violates_friction_circle(self, x: Union[State, np.array], u: Union[State, np.array],
                                  throw: bool = False) -> bool:
@@ -645,13 +699,13 @@ class PointMassNonlinearDynamics(VehicleDynamics):
         """
         u_vals = self.input_to_array(u)[0] if isinstance(u, State) else u
         x_vals = self.state_to_array(x)[0] if isinstance(x, State) else x
-        
+
         a_long = u_vals[0]
         a_lat = u_vals[1] * x_vals[2]
 
         vals_power = a_long ** 2 + a_lat ** 2
         violates = vals_power > self.parameters.longitudinal.a_max ** 2
-        
+
         if throw and violates:
             msg = f'Input violates friction circle constraint!\n' \
                   f'Init state: {x}\n\n Input:{u}'
@@ -677,7 +731,7 @@ class PointMassNonlinearDynamics(VehicleDynamics):
             'velocity': x[2],
             'orientation': x[3]
         }
-        return State(**values, time_step=time_step)
+        return CustomState(**values, time_step=time_step)
 
     def _input_to_array(self, input: State) -> Tuple[np.array, int]:
         """ Overrides VehicleDynamics method. """
@@ -694,7 +748,7 @@ class PointMassNonlinearDynamics(VehicleDynamics):
             'acceleration': u[0],
             'yaw_rate': u[1],
         }
-        return State(**values, time_step=time_step)
+        return CustomState(**values, time_step=time_step)
 
     def occupancy(self, x):
         """Compute the space occupied by the car, used only for the safety layer using reachable sets"""
